section \<open>Demo Theory of 8th Session\<close>

theory Demo08
  imports Main
begin

text \<open>
This theory (importing @{theory Main}) has two purposes:

  \<^enum> it constitutes, together with the corresponding @{file ROOT} file,
    an example of an Isabelle session, and

  \<^enum> illustrates the usage of document antiquotations

In order to produce a PDF run

  @{verbatim "isabelle build -v -D ."}

in the session directory @{dir "."}.

With plain like this `@{text "Cons x xs"}' we can typeset arbitrary terms in Isabelle
inner syntax. If you want to have Isar keywords treated in a special way use
@{verbatim \<open>@{theory_text "..."}\<close>} instead. Consider, for example:

\begin{center}
  @{theory_text "datatype term but also other words"}
\end{center}

\<close>


subsection \<open>Document Antiquotations\<close>

text \<open>
With \<^emph>\<open>document antiquotations\<close> we can achieve many things. We can typeset:

  \<^item> terms (like @{term "[1,2,3,4]::nat list"})
  \<^item> constants (like @{const Cons})
  \<^item> types (like @{typeof "Cons"} and @{typ "nat set"})
  \<^item> type constructors (like @{type "list"})
  \<^item> theorems (like @{thm HOL.refl} and @{lemma "1 = 1" by (rule refl)})
  \<^item> ...
\<close>


subsection \<open>Sets in Isabelle/HOL\<close>

lemma "A \<inter> (B \<union> C) \<subseteq> (A \<inter> B) \<union> (A \<inter> C)"
proof
  fix x assume "x \<in> A \<inter> (B \<union> C)"
  then obtain "x \<in> A" and "x \<in> B \<union> C" ..
  from \<open>x \<in> B \<union> C\<close>
  show "x \<in> (A \<inter> B) \<union> (A \<inter> C)"
  proof
    assume "x \<in> B"
    with \<open>x \<in> A\<close> have "x \<in> A \<inter> B" ..
    then show ?thesis ..
  next
    assume "x \<in> C"
    with \<open>x \<in> A\<close> have "x \<in> A \<inter> C" ..
    then show ?thesis ..
  qed
qed


subsubsection \<open>Cantor's Theorem\<close>

lemma Cantor:
  assumes "f ` A \<subseteq> Pow A"
  shows "\<not> (Pow A \<subseteq> f ` A)"
proof
  assume surj: "Pow A \<subseteq> f ` A"
  let ?B = "{x \<in> A. x \<notin> f x}"
  have "?B \<in> Pow A" by auto
  with surj have "?B \<in> f ` A" by auto
  then obtain y where "y \<in> A" and "f y = ?B" by auto
  from \<open>y \<in> A\<close> have "y \<in> ?B \<longleftrightarrow> y \<notin> f y" by auto
  also have "\<dots> \<longleftrightarrow> y \<notin> ?B"
    unfolding \<open>f y = ?B\<close> ..
  finally have "y \<in> ?B \<longleftrightarrow> y \<notin> ?B" .
  then show False by auto
qed

end
